require 'spec_helper'

describe 'performing a masspay' do
  let(:success_body)  { 'ACK=Success' }
  let(:fail_body)     { 'ACK=Failure&L_ERRORCODE0=10000&L_SHORTMESSAGE0=short_error_message&L_LONGMESSAGE0=long_error_message&L_SEVERITYCODE0=Error&L_ERRORCODE1=20000&L_SHORTMESSAGE1=short_error_message2&L_LONGMESSAGE1=long_error_message2&L_SEVERITYCODE1=Error' }
  let(:response_body) { success_body }

  let(:recipient_one) { Paypal::Masspay::Recipient.new(:recipient_email => 'chunky@example.com', :amount => 10.00) }
  let(:recipient_two) { Paypal::Masspay::Recipient.new(:recipient_email => 'bacon@example.com', :amount => 2.95) }

  before do
    stub_request(:post, "https://api-3t.sandbox.paypal.com/nvp").with(
      :body => {
        "USER"         => "your_paypal_username",
        "PWD"          => "your_paypal_password",
        "SIGNATURE"    => "your_paypal_signature",
        "CURRENCYCODE" => "USD",
        "EMAILSUBJECT" => "Paypal Masspay",
        "L_AMT0"       => "10.0",
        "L_AMT1"       => "2.95",
        "L_EMAIL0"     => "chunky@example.com",
        "L_EMAIL1"     => "bacon@example.com",
        "METHOD"       => "MassPay",
        "RECEIVERTYPE" => "EmailAddress",
        "VERSION"      => "2.3"
    }).to_return(:body => response_body)

    Paypal::Masspay::Configuration.configure do |config|
      config.environment  = :sandbox
      config.username     = 'your_paypal_username'
      config.password     = 'your_paypal_password'
      config.signature    = 'your_paypal_signature'
    end
  end

  context 'adding recipients one at a time' do
    before do
      @masspay = Paypal::Masspay.new(:currency => 'USD', :subject => 'Paypal Masspay') do |bucket|
        bucket.add_recipient(recipient_one)
        bucket.add_recipient(recipient_two)
      end
    end

    context 'when the response was successful' do
      let(:response_body) { success_body }

      it 'sends the correct request to the paypal api and is successful' do
        expect(@masspay).to_not be_delivered
        @masspay.deliver
        expect(@masspay).to be_successful
        expect(@masspay).to be_delivered
      end
    end

    context 'when the response was not successful' do
      let(:response_body) { fail_body }

      it 'sends the correct request to the paypal api and is successful' do
        @masspay.deliver
        expect(@masspay).to_not be_successful
      end

      it 'has 2 errors' do
        @masspay.deliver
        expect(@masspay.errors.count).to eq(2)
      end

      it 'has the correct error codes' do
        @masspay.deliver
        expect(@masspay.errors[0].code).to eq(10000)
        expect(@masspay.errors[1].code).to eq(20000)
      end
    end
  end

  context 'adding recipients in a hash' do
    let(:recipients) { [recipient_one, recipient_two] }

    before do
      @masspay = Paypal::Masspay.new(:currency => 'USD', :subject => 'Paypal Masspay', :recipients => recipients)
    end

    context 'when the response was successful' do
      let(:response_body) { success_body }

      it 'sends the correct request to the paypal api and is successful' do
        @masspay.deliver
        expect(@masspay).to be_successful
      end
    end
  end
end
