require 'spec_helper'

describe Paypal::Masspay do
  let(:recipient_one) { double }
  let(:recipient_two) { double }
  let(:response)    { double(:successful? => successful, :errors => errors) }
  let(:errors)      { [] }

  let(:successful)    { true }
  let(:success_body)  { 'ACK=Success' }
  let(:fail_body)     { 'ACK=Failure&L_ERRORCODE0=10000&L_SHORTMESSAGE0=short_error_message&L_LONGMESSAGE0=long_error_message&L_SEVERITYCODE0=Error&L_ERRORCODE1=20000&L_SHORTMESSAGE1=short_error_message2&L_LONGMESSAGE1=long_error_message2&L_SEVERITYCODE1=Error' }
  let(:response_body) { success_body }

  before do
    allow(Paypal::Masspay::Configuration).to receive(:environment).and_return(:sandbox)
    allow(Paypal::Masspay::Configuration).to receive(:username).and_return("user1")
    allow(Paypal::Masspay::Configuration).to receive(:password).and_return("password1")
    allow(Paypal::Masspay::Configuration).to receive(:signature).and_return("sig1")

    allow(recipient_one).to receive(:get_params).with(0).and_return({"L_EMAIL0" => 'chunky@example.com', "L_AMT0" => 10.0})
    allow(recipient_two).to receive(:get_params).with(1).and_return({"L_EMAIL1" => 'bacon@example.com', "L_AMT1" => 2.95})
    allow(Paypal::Masspay::ResponseParser).to receive(:new).with(response_body).and_return(response)

    allow(Paypal::Masspay::LengthValidator).to receive(:validate).with('USD', 3, :currency).and_return('USD')
    allow(Paypal::Masspay::LengthValidator).to receive(:validate).with('Paypal Masspay', 255, :subject).and_return('Paypal Masspay')

    stub_request(:post, "https://api-3t.sandbox.paypal.com/nvp").with(
      :body => {
        "USER"         => "user1",
        "PWD"          => "password1",
        "SIGNATURE"    => "sig1",
        "CURRENCYCODE" => "USD",
        "EMAILSUBJECT" => "Paypal Masspay",
        "L_AMT0"       => "10.0",
        "L_AMT1"       => "2.95",
        "L_EMAIL0"     => "chunky@example.com",
        "L_EMAIL1"     => "bacon@example.com",
        "METHOD"       => "MassPay",
        "RECEIVERTYPE" => "EmailAddress",
        "VERSION"      => "2.3"
    }).to_return(:body => response_body)
  end

  context 'adding recipients one at a time' do
    before do

      @masspay = Paypal::Masspay.new(:currency => 'USD', :subject => 'Paypal Masspay') do |bucket|
        bucket.add_recipient(recipient_one)
        bucket.add_recipient(recipient_two)
      end
    end

    context 'when the response was successful' do
      let(:response_body) { success_body }
      let(:successful)    { true }
      let(:errors)        { [] }

      it 'sends the correct request to the paypal api and is successful' do
        expect(@masspay).to_not be_delivered
        @masspay.deliver
        expect(@masspay).to be_successful
        expect(@masspay).to be_delivered
      end
    end

    context 'when the response was not successful' do
      let(:response_body) { fail_body }
      let(:successful)    { false }
      let(:errors)        { [ double, double ] }

      it 'sends the correct request to the paypal api and is successful' do
        @masspay.deliver
        expect(@masspay).to_not be_successful
      end

      it 'has 2 errors' do
        @masspay.deliver
        expect(@masspay.errors.count).to eq(2)
      end
    end
  end

  context 'adding recipients in a hash' do
    let(:recipients) { [recipient_one, recipient_two] }

    before do
      @masspay = Paypal::Masspay.new(:currency => 'USD', :subject => 'Paypal Masspay', :recipients => recipients)
    end

    context 'when the response was successful' do
      let(:response_body) { success_body }
      let(:successful)    { true }
      let(:errors)        { [] }

      it 'sends the correct request to the paypal api and is successful' do
        @masspay.deliver
        expect(@masspay).to be_successful
      end
    end
  end
end
